@@ -45,9 +45,11 @@ pub use self::{
     HasRecApp,
     Rec,
     RecApp,
+    RecRow,
     RecX,
     Release,
     SharedRecApp,
+    SharedRecRow,
   },
   session::{
     unsafe_create_session,
