@@ -1,7 +1,12 @@
+use std::marker::PhantomData;
+
 use super::protocol::Protocol;
-use crate::internal::functional::nat::{
-  S,
-  Z,
+use crate::internal::functional::{
+  nat::{
+    S,
+    Z,
+  },
+  row::*,
 };
 
 pub trait RecApp<A>: Sized + 'static
@@ -116,6 +121,26 @@ where
   type Applied = (X::Applied, Y::Applied);
 }
 
+pub trait HasSharedRecApp<F, A>: Send + 'static
+{
+  fn get_applied(self: Box<Self>) -> Box<F::Applied>
+  where
+    F: SharedRecApp<A>;
+}
+
+impl<T, F, A> HasSharedRecApp<F, A> for T
+where
+  F: 'static,
+  A: 'static,
+  T: Send + 'static,
+  F: SharedRecApp<A, Applied = T>,
+{
+  fn get_applied(self: Box<T>) -> Box<T>
+  {
+    self
+  }
+}
+
 impl<X> SharedRecApp<X> for Release
 {
   type Applied = X;
@@ -140,3 +165,33 @@ where
 {
   type Applied = RecX<(), F::Applied>;
 }
+
+pub struct RecRow<R, Row>
+{
+  phantom: PhantomData<(R, Row)>,
+}
+
+pub struct SharedRecRow<R, Row>
+{
+  phantom: PhantomData<(R, Row)>,
+}
+
+impl<R, Row1, Row2, Row3> ToRow for RecRow<R, Row1>
+where
+  R: Send + 'static,
+  Row1: ToRow<Row = Row2>,
+  Row2: RecApp<R, Applied = Row3>,
+  Row3: RowCon,
+{
+  type Row = Row3;
+}
+
+impl<R, Row1, Row2, Row3> ToRow for SharedRecRow<R, Row1>
+where
+  R: Send + 'static,
+  Row1: ToRow<Row = Row2>,
+  Row2: SharedRecApp<R, Applied = Row3>,
+  Row3: RowCon,
+{
+  type Row = Row3;
+}
