@@ -1,6 +1,7 @@
 pub mod base;
 pub mod identity;
 pub mod nat;
+pub mod public;
 pub mod row;
 pub mod type_app;
 
@@ -51,6 +52,7 @@ pub use self::{
     SumApp,
     SumFunctor,
     SumFunctorInject,
+    ToRow,
   },
   type_app::{
     get_applied,
