@@ -1,5 +1,3 @@
-use serde;
-
 use super::{
   structs::*,
   traits::*,
@@ -62,7 +60,7 @@ where
     self
   }
 
-  fn get_sum_borrow<'a>(&'a self) -> &'a Row::Applied
+  fn get_sum_borrow(&self) -> &Row::Applied
   where
     F: TyCon,
     Row: SumApp<F>,
@@ -80,7 +78,18 @@ where
 
 impl RowCon for () {}
 
-impl RowCon for Bottom {}
+impl ToRow for ()
+{
+  type Row = ();
+}
+
+impl<A, R> ToRow for (A, R)
+where
+  A: Send + 'static,
+  R: RowCon,
+{
+  type Row = (A, R);
+}
 
 impl<F, A, R> SumApp<F> for (A, R)
 where
