@@ -8,6 +8,11 @@ pub trait RowCon: Sized + Send + 'static
 {
 }
 
+pub trait ToRow
+{
+  type Row;
+}
+
 pub trait SumApp<F>: RowCon
 where
   F: TyCon,
@@ -34,7 +39,7 @@ pub trait HasSumApp<Row, F>: Send
     F: TyCon,
     Row: SumApp<F>;
 
-  fn get_sum_borrow<'a>(&'a self) -> &'a Row::Applied
+  fn get_sum_borrow(&self) -> &Row::Applied
   where
     F: TyCon,
     Row: SumApp<F>;
