@@ -14,11 +14,12 @@ use crate::internal::{
     AppSum,
     Prism,
     RowCon,
+    ToRow,
   },
   protocol::ExternalChoice,
 };
 
-pub fn choose<N, M, C1, C2, A, B, Row>(
+pub fn choose<N, M, C1, C2, A, B, Row1, Row2>(
   _: N,
   _: M,
   cont: PartialSession<C2, A>,
@@ -28,14 +29,16 @@ where
   C2: Context,
   A: Protocol,
   B: Protocol,
-  Row: RowCon,
-  N: ContextLens<C1, ExternalChoice<Row>, B, Target = C2>,
-  M: Prism<Row, Elem = B>,
+  Row2: RowCon,
+  Row1: Send + 'static,
+  Row1: ToRow<Row = Row2>,
+  N: ContextLens<C1, ExternalChoice<Row1>, B, Target = C2>,
+  M: Prism<Row2, Elem = B>,
 {
   unsafe_create_session(move |ctx1, sender1| async move {
     let (receiver1, ctx2) = N::extract_source(ctx1);
 
-    let choice: AppSum<Row, ()> = M::inject_elem(wrap_type_app(()));
+    let choice: AppSum<Row2, ()> = M::inject_elem(wrap_type_app(()));
 
     let ExternalChoice { sender: sender2 } = receiver1.recv().await.unwrap();
 
