@@ -9,8 +9,9 @@ pub trait SessionInjector<Row, C, A>: Send
   fn inject_session(
     self: Box<Self>,
     session: PartialSession<C, A>,
-  ) -> AppSum<Row, SessionF<C>>
+  ) -> AppSum<Row::Row, SessionF<C>>
   where
     C: Context,
-    A: Protocol;
+    A: Protocol,
+    Row: ToRow;
 }
