@@ -22,11 +22,12 @@ use crate::internal::{
     SumApp,
     SumFunctor,
     SumFunctorInject,
+    ToRow,
   },
   protocol::InternalChoice,
 };
 
-pub fn case<N, C, D, B, Row, SessionSum, InjectSessionSum>(
+pub fn case<N, C, D, B, Row1, Row2, SessionSum, InjectSessionSum>(
   _: N,
   cont1: impl FnOnce(InjectSessionSum) -> SessionSum + Send + 'static,
 ) -> PartialSession<C, B>
@@ -34,18 +35,20 @@ where
   B: Protocol,
   C: Context,
   D: Context,
-  Row: RowCon,
-  Row: ElimSum,
-  Row: SplitRow,
-  Row: SumFunctor,
-  Row: IntersectSum,
-  Row: SumFunctorInject,
-  Row: SumApp<InternalSessionF<N, C, B, Row, D>, Applied = SessionSum>,
-  Row: FlattenSumApp<
-    InjectSessionF<N, C, B, Row, D>,
+  Row1: Send + 'static,
+  Row1: ToRow<Row = Row2>,
+  Row2: RowCon,
+  Row2: ElimSum,
+  Row2: SplitRow,
+  Row2: SumFunctor,
+  Row2: IntersectSum,
+  Row2: SumFunctorInject,
+  Row2: SumApp<InternalSessionF<N, C, B, Row1, D>, Applied = SessionSum>,
+  Row2: FlattenSumApp<
+    InjectSessionF<N, C, B, Row1, D>,
     FlattenApplied = InjectSessionSum,
   >,
-  N: ContextLens<C, InternalChoice<Row>, Empty, Deleted = D>,
+  N: ContextLens<C, InternalChoice<Row1>, Empty, Deleted = D>,
   SessionSum: Send + 'static,
   InjectSessionSum: Send + 'static,
 {
@@ -60,11 +63,11 @@ where
 
     let cont3 = lift_unit_to_session(selector_sum);
 
-    let cont3a = Row::flatten_sum(cont3);
+    let cont3a = Row2::flatten_sum(cont3);
 
     let cont4 = wrap_sum_app(cont1(cont3a));
 
-    let cont5 = Row::intersect_sum(receiver_sum2, cont4);
+    let cont5 = Row2::intersect_sum(receiver_sum2, cont4);
 
     match cont5 {
       Some(cont6) => {
