@@ -20,8 +20,9 @@ where
   A: Protocol,
   B: Protocol,
   C: Context,
+  Row: ToRow,
+  Row: Send + 'static,
   Del: Context,
-  Row: RowCon,
   N: ContextLens<C, InternalChoice<Row>, A, Deleted = Del>,
 {
   pub session: PartialSession<N::Target, B>,
