@@ -15,7 +15,9 @@ pub trait HasInternalSession<N, C, A, B, Row, Del>: Send
     B: Protocol,
     C: Context,
     Del: Context,
-    Row: RowCon,
+    Row: ToRow,
+    Row: Send + 'static,
+    Row::Row: RowCon,
     N: ContextLens<C, InternalChoice<Row>, A, Deleted = Del>;
 }
 
@@ -30,7 +32,9 @@ pub trait NeedInternalSession<N, C, A, B, Row, Del, K>
     B: Protocol,
     C: Context,
     Del: Context,
-    Row: RowCon,
+    Row: ToRow,
+    Row: Send + 'static,
+    Row::Row: RowCon,
     N: ContextLens<C, InternalChoice<Row>, A, Deleted = Del>;
 }
 
