@@ -15,19 +15,22 @@ use crate::internal::{
     wrap_type_app,
     Prism,
     SumApp,
+    ToRow,
   },
   protocol::InternalChoice,
 };
 
-pub fn offer_case<N, C, A, Row>(
+pub fn offer_case<N, C, A, Row1, Row2>(
   _: N,
   cont: PartialSession<C, A>,
-) -> PartialSession<C, InternalChoice<Row>>
+) -> PartialSession<C, InternalChoice<Row1>>
 where
   C: Context,
   A: Protocol,
-  Row: SumApp<ReceiverF>,
-  N: Prism<Row, Elem = A>,
+  Row1: Send + 'static,
+  Row1: ToRow<Row = Row2>,
+  Row2: SumApp<ReceiverF>,
+  N: Prism<Row2, Elem = A>,
 {
   unsafe_create_session(move |ctx, sender1| async move {
     let (sender2, receiver2) = once_channel();
