@@ -6,10 +6,12 @@ pub use self::{
   external::{
     choose,
     offer_choice,
+    InjectExternal,
   },
   internal::{
     case,
     offer_case,
+    InjectInternal,
   },
   run_cont::{
     run_cont,
